@@ -17,16 +17,27 @@
 
 import re
 
+from rql import TypeResolverException
+
 from cubicweb import typed_eid
 from cubicweb.server import hook
 from cubicweb.selectors import is_instance
 
 from cubes.tracker import hooks as tracker
 
-# search for ticket closing instructions #######################################
-
 TICKET_EID_RGX = re.compile('#(\d+)')
 
+def closed_eids(line):
+    words = [w.lower() for w in re.findall('#?\w+', line.strip())]
+    try:
+        idx = words.index('closes')
+    except ValueError:
+        return ()
+    return TICKET_EID_RGX.findall(' '.join(words[idx+1:]))
+
+
+# search for ticket closing instructions #######################################
+
 class SearchRevisionTicketOp(hook.DataOperationMixIn,
                              hook.Operation):
     """search magic words in revision's commit message:
@@ -40,16 +51,14 @@ class SearchRevisionTicketOp(hook.DataOperationMixIn,
         for rev in self.get_data():
             # search for instruction
             for line in rev.description.splitlines():
-                words = [w.lower() for w in re.findall('#?\w+', line.strip())]
-                try:
-                    idx = words.index('closes')
-                except ValueError:
-                    continue
-                for eid in TICKET_EID_RGX.findall(' '.join(words[idx+1:])):
-                    rset = self.session.execute(
-                        'Any X WHERE X eid %(x)s, X concerns P, '
-                        'P source_repository R, R eid %(r)s',
-                        {'x': typed_eid(eid), 'r': rev.repository.eid})
+                for eid in closed_eids(line):
+                    try:
+                        rset = self.session.execute(
+                            'Any X WHERE X eid %(x)s, X concerns P, '
+                            'P source_repository R, R eid %(r)s',
+                            {'x': typed_eid(eid), 'r': rev.repository.eid})
+                    except TypeResolverException:
+                        continue
                     if rset:
                         ticket = rset.get_entity(0, 0)
                         ticket.set_relations(closed_by=rev)
@@ -104,6 +113,7 @@ class PatchLinkedToTicket(hook.Hook):
         ticket = self._cw.entity_from_eid(self.eidto)
         ticket.cw_adapt_to('IWorkflowable').fire_transition_if_possible('start')
 
+
 # attach patch to ticket on addition ###########################################
 
 class PatchRevisionAdded(hook.Hook):
@@ -124,24 +134,26 @@ class PatchRevisionAdded(hook.Hook):
             # and stop at the end of the header
             if line[:4] == 'diff':
                 break
-            words = [w.lower() for w in re.findall('#?\w+', line.strip())]
-            try:
-                idx = words.index('closes')
-            except ValueError:
-                continue
-            for eid in TICKET_EID_RGX.findall(' '.join(words[idx+1:])):
-                rset = self._cw.execute(
-                        'Any X WHERE X eid %(x)s, X concerns P, P source_repository R, R eid %(r)s',
-                        {'x': typed_eid(eid), 'r': source_repo.eid})
+            for eid in closed_eids(line):
+                try:
+                    rset = self._cw.execute(
+                            'Any X WHERE X eid %(x)s, X concerns P, P source_repository R, R eid %(r)s',
+                            {'x': typed_eid(eid), 'r': source_repo.eid})
+                except TypeResolverException:
+                    continue
                 if rset:
                     ticket = rset.get_entity(0,0)
-                    patch.set_relations(patch_ticket=ticket)
+                    # XXX should we drop existing links first?
+                    if all(x.eid != ticket.eid for x in patch.patch_ticket):
+                        patch.set_relations(patch_ticket=ticket)
 
 # security propagation #########################################################
 
 tracker.S_RELS.add('source_repository')
 
 
+# registration control #########################################################
+
 def registration_callback(vreg):
     vreg.register_all(globals().values(), __name__, (RevisionAdded,
                                                      PatchLinkedToTicket,
